@@ -5,12 +5,12 @@ changed.
 @author: Peter Parente
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
 import Base, Constants
@@ -20,8 +20,8 @@ class StateChange(Base.AccessEngineEvent):
   Event that fires when the state of an accessible or item changes.
   
   This class registers its name and whether it should be monitored by default in
-  an L{AEMonitor.EventMonitor} using the L{Constants.registerEventType} function
-  when this module is first imported. The L{AEMonitor.EventMonitor} can use this
+  an L{AEMonitor} using the L{Constants.registerEventType} function
+  when this module is first imported. The L{AEMonitor} can use this
   information to build its menus.
 
   @ivar name: Name of the property that changed
