@@ -11,12 +11,12 @@ grows new modules.
 @author: Brett Clippingdale
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import Constants, Base
 from Constants import NORMAL, HIGH, IMMEDIATE, LAYER_FOCUS, \
@@ -33,6 +33,8 @@ from TableChange import *
 from ScreenChange import *
 from ChooserChange import *
 from TimerAlert import *
+from MouseChange import *
+from SettingChange import * 
 
 def getDefaults():
   '''
