@@ -5,12 +5,12 @@ of a certain event.
 @author: Peter Parente
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import logging
 import Base
@@ -119,12 +119,14 @@ class MonitorCollection(object):
           # initialize each monitor so it is displayed
           m.init()
 
-  def show(self, **kwargs):
+  def show(self, *args, **kwargs):
     '''
     Informs L{AEMonitor}s added via L{add} of information to be buffered. Calls
     L{Base.AEMonitor.show} on all monitors. Monitors raising IOError in show 
     are removed from the L{monitors} list and are no longer notified of events.
     
+    @param args: Information to buffer
+    @type args: list
     @param kwargs: Information to buffer
     @type kwargs: dictionary
     '''
@@ -134,7 +136,7 @@ class MonitorCollection(object):
       m = self.monitors[i]
       try:
         # buffer the given information
-        m.show(**kwargs)
+        m.show(*args, **kwargs)
       except IOError:
         # remove any monitors that have indicated that want no more events
         del self.monitors[i]
