@@ -5,12 +5,12 @@ Defines a threaded proxy for L{AEOutput.Audio} devices.
 @author: Peter Parente
 @organization: IBM Corporation
 @copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
 import threading, Queue, Base
@@ -20,8 +20,6 @@ class AudioThreadProxy(threading.Thread, Base.AEOutput):
   Buffers calls to methods on a L{AEOutput.Audio} device in a secondary thread
   and executes them some time later when that thread runs.
   
-  @todo: PP: what to do with sendStringSync?
-  
   @ivar device: Device on which to invoke send* methods
   @type device: L{AEOutput.Audio}
   @ivar want_stop: Flag indicating a stop is requested
@@ -69,6 +67,16 @@ class AudioThreadProxy(threading.Thread, Base.AEOutput):
     '''
     self.alive = True
     self.start()
+    
+  def getCapabilities(self):
+    '''    
+    Gets the capabilities of the proxied device. This method is called in the
+    context of the caller, not the running thread.
+
+    @return: List of capability names
+    @rtype: list of string
+    '''
+    return self.device.getCapabilities()
 
   def createDistinctStyles(self, num_groups, num_layers):
     '''
@@ -124,56 +132,48 @@ class AudioThreadProxy(threading.Thread, Base.AEOutput):
   def getName(self):
     '''
     Gives the user displayable (localized) name for this output device.
-    Relevant version and device status should be included. This method is called
-    in the context of the caller, not the running thread.
+    Relevant version and device status should be included. This method is 
+    called in the context of the caller, not the running thread.
 
     @return: The localized name for the device
     @rtype: string
     '''
     return self.device.getName()
-
-  def sendString(self, text, style):
-    '''
-    Buffers a call to the device's implementation of L{sendString}.
-
-    @param text: Text to send to the device
-    @type text: string
-    @param style: Style with which this text should be output
-    @type style: L{AEOutput.Style}
-    '''
-    self._put(self.device.sendString, text, style)
-
-  def sendStop(self, style=None):
-    '''
-    Buffers a call to the device's implementation of L{sendStop}.
-
-    @param style: Style indicating which channel on which the stop should be 
-      performed; None indicates stop on all channels
-    @type style: L{AEOutput.Style}
-    '''
-    self._put(self.device.sendStop, style)
-
-  def sendTalk(self, style=None):
-    '''
-    Buffers a call to the device's implementation of L{sendTalk}.
-
-    @param style: Style indicating which channel on which the talk should be 
-      performed; None indicates talk on all channels
-    @type style: L{AEOutput.Style}
-    @raise NotImplementedError: When not overridden in a subclass
-    '''
-    self._put(self.device.sendTalk, style)
+  
+  def send(self, name, value, style=None):
+    '''    
+    Buffers methods to call for known commands in the context of the thread.
     
-  def sendIndex(self, style):
-    '''
-    Buffers a call to the device's implementation of L{sendIndex}.
-
-    @param style: Style indicating which channel on which the index should be 
-      placed
+    @param name: Descriptor of the data value sent
+    @type name: object
+    @param value: Content value
+    @type value: object
+    @param style: Style with which this value should be output
     @type style: L{AEOutput.Style}
+    @return: Return value specific to the given command
+    @rtype: object
     @raise NotImplementedError: When not overridden in a subclass
     '''
-    self._put(self.device.sendIndex, style)
+    if name == Constants.CMD_STOP:
+      self._put(self.device.sendStop, style)
+    elif name == Constants.CMD_TALK:
+       self._put(self.device.sendTalk, style)
+    elif name == Constants.CMD_STRING:
+       self._put(self.device.sendString, value, style)
+    elif name == Constants.CMD_STRING_SYNC:
+      self.device.sendStringSync(value, style)
+    elif name == Constants.CMD_FILENAME:
+      try:
+        # may not be implemented
+        self._put(self.device.sendFilename, value, style)
+      except NotImplementedError:
+        pass
+    elif name == Constants.CMD_INDEX:
+      try:
+        # may not be implemented
+        return self._put(self.device.sendIndex, style)
+      except NotImplementedError:
+        pass
   
   def isActive(self):
     '''
@@ -187,7 +187,8 @@ class AudioThreadProxy(threading.Thread, Base.AEOutput):
   
   def parseString(self, text, style, por):
     '''
-    Parses the string using the implementation in L{Audio.Audio.parseString}.
+    Parses the string using the implementation provided by the proxied device.
+    This method is called in the context of the caller, not the running thread.
     
     @param text: Text to be parsed
     @type text: string
