@@ -7,12 +7,12 @@ interface.
 @author: Brett Clippingdale
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 from POR import POR
 from AEEvent import *
@@ -22,9 +22,38 @@ from DefaultNav import *
 from DefaultEventHandler import DefaultEventHandlerAdapter
 from pyLinAcc import Constants, Interfaces
 import pyLinAcc
+from i18n import _
+import time
 
 FUDGE_PX = 3
 
+def _multiLineCondition(acc):
+  '''
+  Tests whether an accessible should be treated as a multiline text area or
+  a simple one line text control.
+  
+  @param acc: Accessible to test
+  @type acc: L{pyLinAcc.Accessible}
+  '''
+  Interfaces.IText(acc)
+  r = acc.getRole()
+  c = Constants
+  if r in (c.ROLE_TERMINAL, c.ROLE_PARAGRAPH):
+    # terminals and paragraphs are always multiline
+    # note: this does not account for paragraphs that implement hypertext
+    return True
+  elif r in (c.ROLE_PAGE_TAB, c.ROLE_LABEL):
+    # these widgets sometimes show as multiline, but are succinct enough to
+    # treat as single line
+    return False
+  s = acc.getState()
+  if s.contains(c.STATE_SINGLE_LINE):
+    # single line widgets are not multiline (duh)
+    return False
+  # multiline widgets that weren't filtered in previous tests or editable
+  # widgets are considered multiline
+  return s.contains(c.STATE_MULTI_LINE) #or s.contains(c.STATE_EDITABLE)
+
 class TextAccInfoAdapter(DefaultAccInfoAdapter):
   '''
   Overrides L{DefaultAccInfoAdapter} to provide information about lines of text
@@ -47,16 +76,7 @@ class TextAccInfoAdapter(DefaultAccInfoAdapter):
       for this class, False otherwise
     @rtype: boolean
     '''
-    acc = subject.accessible
-    r = acc.getRole()
-    s = acc.getState()
-    c = Constants
-    return (r == c.ROLE_TERMINAL or 
-            r not in (c.ROLE_PAGE_TAB, c.ROLE_LABEL) and 
-            (s.contains(c.STATE_SINGLE_LINE) or
-            s.contains(c.STATE_MULTI_LINE) or
-            s.contains(c.STATE_EDITABLE)) and
-            Interfaces.IText(acc) is not None)
+    return _multiLineCondition(subject.accessible)
   
   @pyLinAcc.errorToLookupError
   def getAccItemText(self):
@@ -69,15 +89,12 @@ class TextAccInfoAdapter(DefaultAccInfoAdapter):
     @rtype: string
     @raise LookupError: When the subject accessible is dead
     @raise IndexError: When the item offset is outside the bounds of the number
-      of lines of text in the subject accessible
-      
-    @todo: LW: Parse returned line for 0xFFFC and replace with content from
-      child when present.
+      of lines of text in the subject accessible     
     '''
     text = Interfaces.IText(self.accessible)
     if self.item_offset is None:
       # gets the accessible name of the parent if item offset is None
-      return super(TextAccInfoAdapter, self).getAccItemText()
+      return super(TextAccInfoAdapter, self).getAccName()
     else:
       # get the line at the provided offset
       line, sOff, eOff = \
@@ -85,7 +102,61 @@ class TextAccInfoAdapter(DefaultAccInfoAdapter):
                                Constants.TEXT_BOUNDARY_LINE_START)
       return unicode(line, 'utf-8')
     return u''
+  
+  #@pyLinAcc.errorToLookupError
+  #def getAccRoleName(self):
+    #'''
+    #Gets the accessible role name of the subject. Ignores the given item offset 
+    #because the default adapter assumes there are no items.
+    
+    #@return: Localized role name
+    #@rtype: string
+    #@raise LookupError: When the accessible object is dead
+    #'''
+    #if self.item_offset is None:
+      ## gets the accessible role name of the parent if item offset is None
+      #return super(TextAccInfoAdapter, self).getAccRoleName()
+    #else:
+      #return _('text line')
 
+  @pyLinAcc.errorToLookupError
+  def getAccVisualExtents(self):
+    '''
+    Gets the visual width and height of a line or the entire text area if
+    item offset is None.
+
+    @return: Width and height extents
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    it = Interfaces.IText(self.accessible)
+    if self.item_offset is None:
+      return super(TextAccInfoAdapter, self).getVisualExtents()
+    line, s, e = text.getTextAtOffset(self.item_offset, 
+                                      Constants.TEXT_BOUNDARY_LINE_START)
+    xs, ys, ws, hs = it.getCharacterExtents(s, Constants.DESKTOP_COORDS)
+    xe, ye, we, he = it.getCharacterExtents(e, Constants.DESKTOP_COORDS)
+    return (xe-xs), (ye-ys)
+
+  @pyLinAcc.errorToLookupError
+  def getAccVisualPoint(self):
+    '''
+    Gets the center of a line or the entire text area if item offset is None.
+
+    @return: x,y coordinates
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    if self.item_offset is None:
+      return super(TextAccInfoAdapter, self).getVisualPoint()
+    # compute the offset in the line
+    off = self.item_offset + self.char_offset
+    acc = self.accessible
+    text = Interfaces.IText(acc)
+    line, s, e = text.getTextAtOffset(off, Constants.TEXT_BOUNDARY_LINE_START)
+    x, y, w, h = text.getCharacterExtents(self.item_offset + self.char_offset,
+                                          Constants.DESKTOP_COORDS)
+    return x, y+h/2
 
 class TextNavAdapter(DefaultNavAdapter):
   '''
@@ -110,16 +181,7 @@ class TextNavAdapter(DefaultNavAdapter):
     for this class, False otherwise
     @rtype: boolean
     '''
-    acc = subject.accessible
-    r = acc.getRole()
-    s = acc.getState()
-    c = Constants
-    return (r == c.ROLE_TERMINAL or 
-            r not in (c.ROLE_PAGE_TAB, c.ROLE_LABEL) and 
-            (s.contains(c.STATE_SINGLE_LINE) or
-            s.contains(c.STATE_MULTI_LINE) or
-            s.contains(c.STATE_EDITABLE)) and
-            Interfaces.IText(acc) is not None)
+    return _multiLineCondition(subject.accessible)
   
   @pyLinAcc.errorToLookupError
   def _getVisibleItemExtents(self, only_visible):
@@ -147,6 +209,10 @@ class TextNavAdapter(DefaultNavAdapter):
     else:
       first = 0
       last = text.characterCount-1
+    if first == last:
+      # nit: protection against bad bounds reporting
+      first = 0
+      last = text.characterCount-1
     return first, last
     
   @pyLinAcc.errorToLookupError
@@ -196,8 +262,8 @@ class TextNavAdapter(DefaultNavAdapter):
   @pyLinAcc.errorToLookupError
   def getPrevItem(self, only_visible=True):
     '''
-    Gets the previous item relative to the one indicated by the L{POR} providing
-    this interface. 
+    Gets the previous item relative to the one indicated by the L{POR} 
+    providing this interface. 
     
     @param only_visible: True when Item in the returned L{POR} must be visible
     @type only_visible: boolean
@@ -257,7 +323,7 @@ class TextNavAdapter(DefaultNavAdapter):
     '''
     acc = self.accessible
     if only_visible:
-      first, last = self._getVisibleItemExtents()
+      first, last = self._getVisibleItemExtents(only_visible)
       return POR(acc, first, 0)
     else:
       return POR(acc, 0, 0)
@@ -332,9 +398,6 @@ class SimpleTextEventHandlerAdapter(DefaultEventHandlerAdapter):
     for this class, False otherwise
     @rtype: boolean
     '''
-    r = subject.getRole()
-    s = subject.getState()
-    c = pyLinAcc.Constants
     return Interfaces.IText(subject) is not None
   
   def _handleTextEvent(self, event, **kwargs):
@@ -359,9 +422,9 @@ class SimpleTextEventHandlerAdapter(DefaultEventHandlerAdapter):
     text = Interfaces.IText(self.subject)
     # get the text of the caret line, it's starting and ending offsets
     line, sOff, eOff = \
-      text.getTextAtOffset(text.caretOffset, Constants.TEXT_BOUNDARY_LINE_START)
+      text.getTextAtOffset(event.detail1, Constants.TEXT_BOUNDARY_LINE_START)
     # create a POR with start offset of the line and relative caret offset
-    por = POR(self.subject, sOff, text.caretOffset - sOff)
+    por = POR(self.subject, sOff, event.detail1 - sOff)
 
     # create Caret event with current POR, text added/removed, where it changed,
     # and whether it was inserted.
@@ -451,18 +514,10 @@ class TextEventHandlerAdapter(SimpleTextEventHandlerAdapter):
     @param subject: Accessible to test
     @type subject: L{pyLinAcc.Accessible}
     @return: True when the subject meets the condition named in the docstring
-    for this class, False otherwise
+      for this class, False otherwise
     @rtype: boolean
     '''
-    r = subject.getRole()
-    s = subject.getState()
-    c = pyLinAcc.Constants
-    return (r == c.ROLE_TERMINAL or 
-            r not in (c.ROLE_PAGE_TAB, c.ROLE_LABEL) and 
-            (s.contains(c.STATE_SINGLE_LINE) or
-            s.contains(c.STATE_MULTI_LINE) or
-            s.contains(c.STATE_EDITABLE)) and
-            Interfaces.IText(subject) is not None)
+    return _multiLineCondition(subject)
       
   def _handleFocusEvent(self, event, **kwargs):
     '''
@@ -477,6 +532,7 @@ class TextEventHandlerAdapter(SimpleTextEventHandlerAdapter):
     @return: L{AEEvent.FocusChange} and L{AEEvent.CaretChange}
     @rtype: tuple of L{AEEvent}
     '''
+    kwargs['focused'] = True
     text = Interfaces.IText(self.subject)
     char_offset = text.caretOffset
     # get the text of the caret line, it's starting and ending offsets
