@@ -8,12 +8,12 @@ contents of the profile views using information from the L{UIRegistrar}.
 @author: Peter Parente
 @organization: IBM Corporation
 @copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import pygtk
 pygtk.require('2.0')
@@ -23,6 +23,7 @@ from pyLinAcc import Atk
 import AEChooser, AEState
 from i18n import _, DOMAIN
 from GTKUIEView import *
+import weakref
 
 __uie__ = dict(kind='chooser')
 
@@ -32,32 +33,47 @@ class WidgetFactory(object):
   one public method for creating new widgets. All other methods are protected
   for internally handling widget events and updating their correspoding 
   settings.
+  
+  @ivar chooser: Chooser that is requesting the creation of widgets
+  @type chooser: L{AEChooser}
   '''
-  @classmethod
-  def _updateBool(cls, widget, setting):
+  def __init__(self, chooser):
+    '''
+    Stores a reference to the chooser creating this factory.
+    
+    @param chooser: Chooser that is requesting the creation of widgets
+    @type chooser: L{AEChooser}
+    '''
+    self.chooser = weakref.proxy(chooser)
+    
+  def _signalSetting(self, setting):
+    self.chooser._signalSetting(setting.context, setting.variable, 
+                                setting.value)
+  
+  def _updateBool(self, widget, setting):
     setting.value = widget.get_active()
+    self._signalSetting(setting)
     
-  @classmethod
-  def _updateNumeric(cls, adj, setting):
+  def _updateNumeric(self, adj, setting):
     setting.value = adj.get_value()
+    self._signalSetting(setting)
     
-  @classmethod
-  def _updateFilename(cls, widget, setting):
+  def _updateFilename(self, widget, setting):
     fn = widget.get_filename()
     if fn is not None:
       setting.value = fn
+    self._signalSetting(setting)
     
-  @classmethod
-  def _updateString(cls, widget, setting):
+  def _updateString(self, widget, event, setting):
     setting.value = widget.get_text()
+    self._signalSetting(setting)
 
-  @classmethod
-  def _updateChoice(cls, widget, setting):
+  def _updateChoice(self, widget, setting):
     index = widget.get_active()
     setting.value = setting.values[index]
+    self._signalSetting(setting)
   
-  @classmethod
-  def create(cls, setting):
+  def create(self, setting):
     widget = None
     if isinstance(setting, AEState.Group):
       # another group
@@ -68,21 +84,19 @@ class WidgetFactory(object):
       # boolean
       widget = gtk.CheckButton()
       widget.set_active(setting.value)
-      widget.connect('toggled', cls._updateBool, setting)
+      widget.connect('toggled', self._updateBool, setting)
       # set the accessible name
       Atk.setNameDesc(widget, name=setting.label)
     elif isinstance(setting, AEState.FilenameSetting):
       # filename
       widget = gtk.FileChooserButton(setting.label)
       widget.set_filename(setting.value)
-      widget.connect('selection-changed', cls._updateFilename, setting)
+      widget.connect('selection-changed', self._updateFilename, setting)
     elif isinstance(setting, AEState.StringSetting):
       # string
       widget = gtk.Entry()
       widget.set_text(setting.value)
-      # @todo: PP: what signal? changed? appropriate for all updates? what 
-      # about intermediate invalid values?
-      widget.connect('changed', cls._updateString, setting)
+      widget.connect('focus-out-event', self._updateString, setting)
     elif isinstance(setting, AEState.NumericSetting):
       # numeric or range
       if setting.precision == 0:
@@ -97,19 +111,19 @@ class WidgetFactory(object):
         widget = gtk.SpinButton(adj)
       widget.set_digits(setting.precision)
       widget.set_value(setting.value)
-      adj.connect('value-changed', cls._updateNumeric, setting)
+      adj.connect('value-changed', self._updateNumeric, setting)
     elif isinstance(setting, AEState.EnumSetting):
       # enum
       widget = gtk.combo_box_new_text()
       map(widget.append_text, setting.labels)
       widget.set_active(setting.values.index(setting.value))
-      widget.connect('changed', cls._updateChoice, setting)
+      widget.connect('changed', self._updateChoice, setting)
     elif isinstance(setting, AEState.ChoiceSetting):
       # choice
       widget = gtk.combo_box_new_text()
       map(widget.append_text, setting.values)
       widget.set_active(setting.values.index(setting.value))
-      widget.connect('changed', cls._updateChoice, setting)
+      widget.connect('changed', self._updateChoice, setting)
     else:
       raise NotImplementedError
     # create a label
@@ -122,6 +136,8 @@ class SettingsChooser(AEChooser.AEChooser):
   LSR settings dialog for configuring Perks, devices, profiles, and other 
   system settings.
   
+  @ivar factory: Factory used to create gtk widget for settings
+  @type factory: L{WidgetFactory}
   @ivar dialog: Dialog widget for configuring LSR settings
   @type dialog: gtk.Dialog
   @ivar system: LSR system wide settings
@@ -208,6 +224,9 @@ class SettingsChooser(AEChooser.AEChooser):
     @param profile: Name of the profile
     @type profile: string
     '''
+    # create a widget factory
+    self.factory = WidgetFactory(self)
+    
     # store group objects
     self.system = system
     self.perks = perks
@@ -220,11 +239,7 @@ class SettingsChooser(AEChooser.AEChooser):
 
     # get the dialog widget
     self.dialog = source.get_widget('main dialog')
-    try:
-      # try to bring the window to the foreground if this feature is supported
-      self.dialog.window.set_user_time(timestamp)
-    except AttributeError:
-      pass
+    self.activate(timestamp)
     
     # get widget references
     self.tips = gtk.Tooltips()
@@ -259,6 +274,15 @@ class SettingsChooser(AEChooser.AEChooser):
     self._createProfileView(associated, installed)
     self._createSystemSettingsView()
     
+  def activate(self, timestamp, **kwargs):
+    '''Try to bring the window to the foreground.'''
+    try:
+      # try to bring the window to the foreground if this feature is supported
+      self.dialog.window.set_user_time(timestamp)
+    except AttributeError:
+      pass
+    self.dialog.present()
+    
   def _createCustomWidget(self, glade, function_name, widget_name, *args):
     '''
     Creates a custom widget. Invoked during processing by glade.
@@ -458,7 +482,7 @@ class SettingsChooser(AEChooser.AEChooser):
     group = list(group)
     group.reverse()
     for row, setting in enumerate(group):
-      widget, label = WidgetFactory.create(setting)
+      widget, label = self.factory.create(setting)
       if label is None:
         # if the widget is another group, recurse
         layout = self._populateSection(setting, viewport)
@@ -562,4 +586,11 @@ class SettingsChooser(AEChooser.AEChooser):
     @type devices: dictionary of string : L{AEState.Setting.Group}
     '''
     self.devices = devices
-    self._createDeviceSettingsView()
\ No newline at end of file
+    self._createDeviceSettingsView()
+  
+  def updateSetting(self, state, name, value):
+    '''
+    @todo: PP: need to get a reference to the panel showing the given state,
+      and the widget representing the named setting
+    '''
+    print 'chooser updating', state, name, value
