@@ -2,53 +2,93 @@
 Defines a device that uses the IBM TTS synthesizer via the pyibmtts package.
 pyibmtts can be obtained from U{http://ibmtts-sdk.sf.net}.
 
-@var IBMSpeechStyle: Singleton default speech style for all IBM-TTS flyweights
-@type IBMSpeechStyle: L{AEOutput.AudioStyleDefault}
-
 @author: Larry Weiss
 @author: Peter Parente
 @author: Brett Clippingdale
 @organization: IBM Corporation
 @copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import AEOutput
 import pyibmtts
 from i18n import _
 
-class IBMSpeechStyle(AEOutput.AudioStyleDefault):
+__uie__ = dict(kind='device')
+
+class IBMSpeechStyle(AEOutput.StyleFlyweight):
+  '''
+  Overrides the base L{AEOutput.Style.StyleFlyweight} class to provide style
+  settings relative to the default style for this device.
+  '''
+  ADDITIVE = ('Volume', 'Rate', 'Pitch')
+  Rate = 0
+  Volume = 0
+  Pitch = 0
+  
+  def getSettings(self):
+    '''
+    Gets configurable relative settings affecting output from this device for
+    particular types of information.
+    
+    @todo: PP: support in a later version
+    
+    @return: Group of all configurable settings per semantic information type
+    @rtype: L{AEState.Setting.Group}
+    '''
+    raise NotImplementedError
+
+class IBMSpeechStyleDefault(AEOutput.AudioStyleDefault):
   '''
   Overrides the base L{AEOutput.Style.AudioStyleDefault} class, filling in 
-  fields for supported style properties with their appropriate values. After
-  this class is defined, it is immediately turned into a singleton so it may
-  be used ubiquitously as the default style for this device.
+  fields for supported style properties with their appropriate values.
   '''
   MaxVoice = 8
   MinVoice = 1
   MaxVolume = 65535
-  MinVolume = 0
-  Volume = 26987
+  MinVolume = 1
+  Volume = 58982 # about 90%
   MinPitch = 40
   MaxPitch = 442
+  Pitch = 220
   MaxRate = 1297
   MinRate = 176
   Rate = 250
-IBMSpeechStyle = IBMSpeechStyle()  #instantiate a singleton overriding the class
+  Voice = 1
 
+  def getSettings(self):
+    '''
+    Gets configurable absolute settings affecting all output from this device.
+    
+    @return: Group of all configurable settings
+    @rtype: L{AEState.Setting.Group}
+    '''
+    g = self._newGroup()
+    a = g.newGroup(_('Speech'))
+    a.newRange('Pitch', _('Pitch'), self.MinPitch, self.MaxPitch, 0,
+               _('Baseline voice pitch in Hertz'))
+    a.newRange('Rate', _('Rate'), self.MinRate, self.MaxRate, 0,
+               _('Speech rate in words per minute'))
+    a.newPercent('Volume', _('Volume'), self.MinVolume, self.MaxVolume, 0,
+                 _('Speech volume as a percentage'))
+    # generate a group for standard word parsing settings
+    self._newWordGroup(g)
+    return g
+  
 class IBMSpeech(AEOutput.Audio):
   '''
-  Defines a class to send output from LSR to IBM TTS using the pyibmtts wrapper
-  provided with ibmtts-sdk U{http://ibmtts-sdk.sf.net}.
-
-  To be compliant with LSR requirements, this class implements the interface
-  defined in the L{AEOutput.Audio} class.
+  IBM TTS via the pyibmtts wrapper.
+ 
+  @ivar last_style: Last style object to be applied to output
+  @type last_style: L{AEOutput.Style}
   '''    
   USE_THREAD = False
+  DEFAULT_STYLE = IBMSpeechStyleDefault
+  FLYWEIGHT_STYLE = IBMSpeechStyle
   
   def _localize(self, text):
     '''
@@ -65,7 +105,7 @@ class IBMSpeech(AEOutput.Audio):
     '''
     try:
       # kludge!
-      return text.encode('latin-1', 'replace')
+      return text.encode('latin-1', 'ignore')
     except (TypeError, UnicodeDecodeError), e:
       return text
 
@@ -75,6 +115,7 @@ class IBMSpeech(AEOutput.Audio):
     
     @raise AEOutput.InitError: When the device can not be initialized
     '''
+    self.last_style = None
     try:
       self.tts = pyibmtts.Engine()
     except pyibmtts.ECIException:
@@ -82,25 +123,30 @@ class IBMSpeech(AEOutput.Audio):
     # use real units and start at some speedy rate
     self.tts.realWorldUnits = True
       
-  def createDistinctStyles(self, num_styles):
+  def createDistinctStyles(self, num_groups, num_layers):
     '''
-    Creates distinct output styles differentiated by voice and  pitch up to 
-    the maximum voices given by the default style for this device. Styles are
-    repeated beyond that threshold.
-      
-    @param num_styles: Number of styles that the requestor would like the
-      device to initialize
-    @type num_styles: integer
+    Creates distinct styles following the guidelines set forth in the base
+    class. Only distinguishes groups using voice, not layers.
+    
+    @param num_groups: Number of sematic groups the requestor would like to
+      represent using distinct styles
+    @type num_groups: integer
+    @param num_layers: Number of content origins (e.g. output originating from
+      a background task versus the focus) the requestor would like to represent
+      using distinct styles
+    @type num_layers: integer   
     @return: Styles
     @rtype: list of L{AEOutput.Style}
     ''' 
     styles = []
-    for i in xrange(1, num_styles+1):
-      s = AEOutput.Style(IBMSpeechStyle)
+    for i in xrange(1, num_groups*num_layers+1):
+      s = IBMSpeechStyle(self.default_style)
       v_num = (i) % s.MaxVoice
       v = self.tts.getVoice(v_num)
       s.Voice = v_num
-      s.Pitch = v.pitchBaseline
+      s.Rate = 0
+      s.Pitch = v.pitchBaseline-self.default_style.Pitch
+      s.Volume = v.volume-self.default_style.Volume
       styles.append(s)
     return styles
 
@@ -144,7 +190,8 @@ class IBMSpeech(AEOutput.Audio):
       style change should be applied
     @type style: integer
     '''
-    if style is not None:
+    if style.isDirty() or self.last_style != style:
+      self.last_style = style
       self._applyStyle(style)
     try:
       self.tts.addText(self._localize(text))
@@ -174,7 +221,7 @@ class IBMSpeech(AEOutput.Audio):
     '''
     return 'IBM-TTS %s' % pyibmtts.getVersion()
   
-  def sendIndex(self, marker, style):
+  def sendIndex(self, style):
     '''
     Sends an index marker to the device driver. The driver should notify the
     device when the marker has been reached during output.
@@ -183,9 +230,6 @@ class IBMSpeech(AEOutput.Audio):
 
     @param style: Style indicating channel in which the marker will be appended
     @type style: L{AEOutput.Style}
-    @param marker: The value that should be returned when output has been 
-      processed up to this point
-    @type marker: integer
     '''
     raise NotImplementedError
 
@@ -202,28 +246,19 @@ class IBMSpeech(AEOutput.Audio):
       v = self.tts.getVoice(style.Voice)
       v = self.tts.setActiveVoice(v)
     except pyibmtts.ECIException:
-      # just quit if we can't even get the voice to which style params should be
-      # applied
+      # just quit if we can't even get the voice to which style params should 
+      # be applied
       return
+    # restore voice independent parameters
     try:
-      # restore voice independent parameters
-      v.speed = style.Rate
+      v.speed = min(max(style.Rate, style.MinRate), style.MaxRate)
     except pyibmtts.ECIException:
       pass
     try:
-      v.volume = style.Volume
+      v.volume = min(max(style.Volume, style.MinVolume), style.MaxVolume)
     except pyibmtts.ECIException:
       pass
     try:
-      v.pitchBaseline = style.Pitch
+      v.pitchBaseline = min(max(style.Pitch, style.MinPitch), style.MaxPitch)
     except pyibmtts.ECIException:
-      pass    
-    
-  def getDefaultStyle(self):
-    '''
-    Gets the singleton default style for this device.
-    
-    @return: Default style of the device
-    @rtype: L{AEOutput.AudioStyleDefault}
-    '''
-    return IBMSpeechStyle
+      pass  
