@@ -6,12 +6,12 @@ response to L{AEEvent}s and L{AEInput.Gesture}s.
 @author: Pete Brunet
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import logging, weakref, os
 import Task, AEInput, AEEvent, LSRInterfaces, UIElement
@@ -70,12 +70,14 @@ class Perk(Task.Tools.All, UIElement.UIE):
     L{AEInput.GestureList} sequence on some L{AEInput} device. The name should
     be hashed into the L{named_tasks} dictionary to fully resolve the L{Task}.
   @type commands: weakref.WeakValueDictionary of (GestureList : string)
-  @ivar state: Settings for this L{Perk}. Defaults to a dummy object that can
-    be used to store state information that is not configurable or 
-    persistable until registerState is called.
-  @type state: L{AEState.AEState}
-  @ivar choosers: L{Task}s to execute in response to a L{AEEvent.ChooserChange}
-  @type choosers: weakref.WeakValueDictionary of (Chooser : Task)
+  @ivar state: Settings for this L{Perk}. Defaults to an empty L{PerkState} 
+    object that can be used to store state information that is not 
+    configurable or persistable unless L{STATE} is specified.
+  @type state: L{PerkState}
+  @ivar out_caps: Capabilities set by L{setIdealOutput} as ideal for output
+    from this L{Perk}. These are used to "late bind" a device for use during
+    execution of this L{Perk} and its L{Task}s.
+  @type out_caps: list of string
   '''
   STATE = PerkState
   
@@ -84,6 +86,9 @@ class Perk(Task.Tools.All, UIElement.UIE):
     Creates empty dictionaries for named, event, command, and chooser L{Task}s.
     Create a state object for storage using the class in L{STATE} in case the
     L{Perk} hasn't previously persisted its settings on disk.
+
+    @note: L{Perk} writers should override L{init} to do initialization, not
+    this method.
     '''
     super(Perk, self).__init__(*args, **kwargs)
     self.named_tasks = {}
@@ -93,10 +98,14 @@ class Perk(Task.Tools.All, UIElement.UIE):
     # instantiate an instance of the STATE class; it may be filled with data
     # when preInit is called with state already persisted to disk
     self.state = self.STATE()
+    self.out_caps = []
     
   def preInit(self, *args, **kwargs):
     '''
     Override pre-initialization to load persisted state from disk.
+
+    @note: L{Perk} writers should override L{init} to do initialization, not
+    this method.
     '''
     # let the tools preInit run first
     super(Perk, self).preInit(*args, **kwargs)
@@ -110,6 +119,9 @@ class Perk(Task.Tools.All, UIElement.UIE):
   def postClose(self):
     '''
     Frees all L{Task}s managed by this L{Perk}.
+
+    @note: L{Perk} writers should override L{close} to do finialization, not
+    this method.
     '''
     # let the tools postClose run second
     super(Perk, self).postClose()
@@ -125,6 +137,30 @@ class Perk(Task.Tools.All, UIElement.UIE):
     self.keyed_tasks = {}
     self.event_tasks = {}
     self.commands = {}
+    
+  def setIdealOutput(self, *capabilities):
+    '''
+    Sets the ideal device capabilities to be used to locate a device for all 
+    output methods invoked from the caller's L{Perk} module, including the 
+    Perk class itself and all of its registered L{Task}s. The capabilities list
+    should include strings naming L{AEOutput} interfaces ("audio" and/or 
+    "braille") at present.
+    
+    @param capabilities: Names of capabilities required on the device
+    @type capabilities: list of string    
+    '''
+    self.out_caps = capabilities
+    
+  def getIdealOutput(self):
+    '''        
+    Gets the ideal device capabilities to locate a device to be used by all
+    output methods invoked from the caller's L{Perk} module, including the Perk
+    class itself and all of its registered L{Task}s.
+    
+    @return: Names of capabilities set as ideal for a default output device
+    @rtype: list of string
+    '''
+    return self.out_caps
   
   def getState(self):
     '''
@@ -134,7 +170,7 @@ class Perk(Task.Tools.All, UIElement.UIE):
     @rtype: L{AEState.AEState}
     '''
     return self.state
-  
+
   def _initTask(self, task):
     '''
     Initializes a L{Task} by calling L{Task.Task.preInit} and 
@@ -218,7 +254,6 @@ class Perk(Task.Tools.All, UIElement.UIE):
     if self.tier.getKeyedTask(cid) is not None:
       raise ValueError('Chooser Task already registered')
     self._initTask(task)
-    # the Task has already been initialized, so don't do it again
     self.keyed_tasks[cid] = task
     self.tier.addPerkRef(cid, self)
     
@@ -295,7 +330,11 @@ class Perk(Task.Tools.All, UIElement.UIE):
     # initialize the Task
     self._initTask(task)
     # opt: inform the Tier of the kind of event that is desired by this Perk
-    self.tier.setEventInterest(kind, True)
+    try:
+      self.tier.setEventInterest(kind, True)
+    except KeyError:
+      # ignore unknown event types, could be internal
+      pass
     for layer, val in d.items():
       if val:
         # get the list of Task registered for this type and layer or insert an
@@ -407,7 +446,11 @@ class Perk(Task.Tools.All, UIElement.UIE):
     task.postClose()
     # opt: inform the Tier of the kind of event that is no longer
     # desired by this Perk
-    self.tier.setEventInterest(kind, False)
+    try:
+      self.tier.setEventInterest(kind, False)
+    except KeyError:
+      # ignore unknown event types, could be internal
+      pass
     for layer, val in d.items():
       if val:
         # get the list of Task registered for this type and layer or insert an
