@@ -7,15 +7,16 @@ Defines the default user interface for LSR.
 @author: Brett Clippingdale
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import Perk, Task, LSRConstants
 from POR import POR
+import unicodedata
 from i18n import _
 
 __uie__ = dict(kind='perk', all_tiers=True)
@@ -38,6 +39,8 @@ class DefaultPerkState(Perk.PerkState):
   OnlyVisible = True
   Wrap = True
   RepeatRole = False
+  WordEcho = False
+  CharEcho = True
   
   def getSettings(self):
     '''
@@ -47,17 +50,24 @@ class DefaultPerkState(Perk.PerkState):
     @rtype: L{AEState.Setting.Group}
     '''
     g = self._newGroup()
-    g.newBool('OnlyVisible', _('Review visible items only?'), 
-              _('When set, reviewing only includes visible items. Otherwise, '
-                'invisible items are also included.'))
-    g.newBool('Wrap', _('Wrap pointer across items?'),
-            _('When set, next and previous, word and character navigation '
-              'can cross item boundaries. Otherwise, first and last, word '
-              'and character are announced instead.'))
-    g.newBool('RepeatRole', _('Always say role?'),
+    e = g.newGroup(_('Echo'))
+    e.newBool('WordEcho', _('Echo words?'), 
+              _('When set, entire words are spoken when editing text.'))
+    e.newBool('CharEcho', _('Echo characters?'), 
+            _('When set, individual characters are spoken when editing text.'))
+    v = g.newGroup(_('Verbosity'))
+    v.newBool('RepeatRole', _('Always say role?'),
             _('When set, the role name of every widget is announced. '
               'Otherwise, the role name is only announced when a new role is '
               'encountered.'))
+    r = g.newGroup(_('Review mode'))
+    r.newBool('OnlyVisible', _('Review visible items only?'), 
+              _('When set, reviewing only includes visible items. Otherwise, '
+                'invisible items are also included.'))
+    r.newBool('Wrap', _('Wrap pointer across items?'),
+            _('When set, next and previous, word and character navigation '
+              'can cross item boundaries. Otherwise, first and last, word '
+              'and character are announced instead.'))
     return g
   
 class DefaultPerk(Perk.Perk):
@@ -73,6 +83,15 @@ class DefaultPerk(Perk.Perk):
   @type last_level: integer
   @ivar last_container: Stores the previously announced container name
   @type last_container: string
+  @ivar last_sel_len: Length of the selected text when the last selection event
+    was received
+  @type last_sel_len: integer
+  @ivar last_row: Index of the last row activated
+  @type last_row: integer
+  @ivar last_col: Index of the last column activated
+  @type last_col: integer
+  @ivar last_count: Last count of selected characters
+  @type last_count: integer 
   '''
   # defines the class to use for state information in this Perk
   STATE = DefaultPerkState
@@ -83,6 +102,8 @@ class DefaultPerk(Perk.Perk):
     property change events. Registers named L{Task}s that can be mapped to 
     input gestures.
     '''
+    # set an audio device as the default output
+    self.setPerkIdealOutput('audio')
     # register event handlers
     self.registerTask(HandleFocusChange('default focus'))
     self.registerTask(HandleViewChange('default view'))
@@ -108,23 +129,31 @@ class DefaultPerk(Perk.Perk):
     self.registerTask(FocusToPointer('focus to pointer'))
     self.registerTask(PointerToFocus('pointer to focus'))
     self.registerTask(ReadTop('read top'))
+    self.registerTask(ReadTextColor('read text color'))
     self.registerTask(ReadTextAttributes('read text attributes'))
     self.registerTask(ReadDescription('read description'))
     self.registerTask(ReadMessage('read message'))
     self.registerTask(ReadNewLabel('read new label'))
     self.registerTask(ReadNewRole('read new role'))
     self.registerTask(ReadNewLevel('read new level'))
+    self.registerTask(ReadNewHeaders('read new headers'))
     self.registerTask(ReadNewContainer('read new container'))
+    self.registerTask(ReadItemIndex('read item index'))
     self.registerTask(ReadItemDetails('read item details'))
     self.registerTask(ReadAll('read all'))
     
+    # cyclic tasks
+    self.registerTask(Task.CyclicInputTask('cycle text attributes',
+                                           'read text color',
+                                           'read text attributes'))
+    
     # stateful variables; OK to access from other Task classes in this module
     self.resetLasts()
   
   def resetLasts(self, container=''):
     '''
-    Resets variables tracking last announced role, level, container as well as
-    the last caret position.
+    Resets variables tracking the last container, caret position, tree level, 
+    selection length, row/col offsets, and row/col count on view change.
     
     @param container: Last container name announced
     @type container: string
@@ -133,6 +162,10 @@ class DefaultPerk(Perk.Perk):
     self.last_caret = POR(item_offset=0)
     self.last_role = None
     self.last_level = None
+    self.last_sel_len = 0
+    self.last_row = None
+    self.last_col = None
+    self.last_count = False
 
 class Stop(Task.InputTask):
   '''
@@ -155,8 +188,12 @@ class IncreaseRate(Task.InputTask):
     rate = self.getRate()
     max_rate = self.getMaxRate()
     if rate < max_rate:
-      self.setRate(rate+10)
-      self.sayInfo(text=(rate+10), template=('rate %d'))
+      rate += 10
+      self.setRate(rate)
+      self.sayInfo(text=(rate), template=('rate %d'))
+      # notify about a change in setting in case the settings dialog is open
+      style = self.getStyle()
+      self.signalSetting(style, 'Rate', rate)
     else:
       self.sayInfo(text=_('maximum rate'))
 
@@ -171,8 +208,12 @@ class DecreaseRate(Task.InputTask):
     rate = self.getRate()
     min_rate = self.getMinRate()
     if rate > min_rate:
-      self.setRate(rate-10)
-      self.sayInfo(text=rate-10, template=_('rate %d'))
+      rate -= 10
+      self.setRate(rate)
+      self.sayInfo(text=rate, template=_('rate %d'))
+      # notify about a change in setting in case the settings dialog is open
+      style = self.getStyle()
+      self.signalSetting(style, 'Rate', rate)
     else:
       self.sayInfo(text=_('minimum rate'))
       
@@ -409,8 +450,109 @@ class PointerToFocus(Task.InputTask):
     self.task_por = self.getAccFocus()
     # stop all output and execute the default focus and selector handlers
     self.doTask('default focus', por=self.task_por, gained=True)
-    self.doTask('read item details')
+    self.doTask('read item details')    
+
+class ReadItemIndex(Task.Task):
+  '''
+  Reads the row and/or column indices of the current item followed by the 
+  total number of items. The total number are only announced if a new control
+  has received focus.
+  '''
+  def execute(self, por=None, **kwargs):
+    '''    
+    Speaks the row and column indices of the given L{POR} or the L{task_por} if
+    no L{POR} is given. Also speaks the total number of items if it has not
+    been announced previously.
     
+    @todo: PP: improve non-table cases where index is important (e.g. all menu
+      items, radio groups)
+    
+    @param por: Point of regard to an object possibly having an index of 
+      interest
+    @type por: L{POR}
+    '''
+    # get the number of rows and cols
+    exts = self.getAccTableExtents(por)
+    if exts is None:
+      if self.hasAccRole('menu item'):
+        # accessible that does not implement the table interface
+        i = self.getAccIndex(por)
+        # get the parent count
+        parent = self.getParentAcc()
+        count = self.getAccCount(parent)
+        if not self.perk.last_count:
+          text = (i+1, count)
+          template = _('item %d of %d')
+        else:
+          text = i+1
+          template = _('item %d')
+        # first is 1, not 0
+        self.sayIndex(text=text, template=template)
+        self.perk.last_count = True
+    else:
+      rows, cols = exts
+      # always get the current row number
+      row = self.getAccRow(por)
+      print cols
+      if cols <= 1:
+        # lists
+        if not self.perk.last_count:
+          text = (row+1, rows)
+          template = _('item %d of %d')
+        else:
+          text = row+1
+          template = _('item %d')
+        # first is 1, not 0
+        self.sayIndex(text=text, template=template)
+      else:
+        # tables
+        col = self.getAccColumn(por)
+        if not self.perk.last_count:
+          try:
+            text = (row+1, col+1, rows, cols)
+          except TypeError:
+            # not in a cell
+            return
+          template = _('item %d, %d, of %d, %d')
+        else:
+          try:
+            text = (row+1, col+1)
+          except TypeError:
+            # not in a cell
+            return
+          template = _('item %d, %d')
+          # first is 1,1 , not 0,0
+        self.sayIndex(text=text, template=template)
+      self.perk.last_count = True
+
+class ReadNewHeaders(Task.Task):
+  '''
+  Reads the row and/or column headers if they differ from the last ones
+  announced.
+  '''
+  def execute(self, por=None, **kwargs):
+    '''    
+    Speaks the row and column headers of the given L{POR} or the L{task_por} if
+    no L{POR} is given. Only makes an announcement if the last row/col
+    encountered by this L{Task} or L{ReadNewHeaders} is different from the
+    current.
+    
+    @param por: Point of regard to a table cell possibly having a header
+    @type por: L{POR}
+    '''
+    row = self.getAccRow(por)
+    if row is not None and row != self.perk.last_row:
+      header = self.getAccRowHeader(por)
+      if header:
+        self.saySection(text=header, template=_('row %s'))
+      self.perk.last_row = row
+    col = self.getAccColumn(por)
+    if col is not None and col != self.perk.last_col:
+      header = self.getAccColumnHeader(por)
+      if header:
+        self.saySection(text=header, template=_('column %s'))
+      self.perk.last_col = col
+
 class ReadNewRole(Task.Task):
   '''
   Reads the role of a widget if it is different from the last role read.
@@ -445,7 +587,8 @@ class ReadNewLevel(Task.Task):
     '''
     level = self.getAccLevel(por)
     if level is not None and level != self.perk.last_level:
-      self.sayLevel(text=level, template=_('level %d'))
+      # root is 1
+      self.sayLevel(text=level+1, template=_('level %d'))
       self.perk.last_level = level
 
 class ReadNewLabel(Task.Task):
@@ -584,15 +727,81 @@ class ReadTop(Task.InputTask):
   def execute(self, **kwargs):
     self.stopNow()
     self.sayWindow()
+    
+class _BaseAttributes(Task.InputTask):
+  '''
+  Base class for other Tasks that will read text attributes.
+  '''
+  def _getAttrs(self):
+    '''
+    Reusable method for getting default and current text attributes and 
+    merging them into one dictionary.
+    
+    @return: Dictionary of name/value attribute pairs
+    @rtype: dictionary
+    '''
+    attrs = self.getAccAllTextAttrs()
+    dattrs = self.getAccDefTextAttrs()
+    
+    if not attrs and not dattrs:
+      # no atttribute information
+      self.sayInfo(text=_('text attributes unavailable'))
+      return
+    elif not attrs:
+      # only default attribute information
+      attrs = dattrs
+    elif not dattrs:
+      # only current attribute information
+      pass
+    else:
+      # overwrite default with current
+      attrs = dattrs.update(attrs)
+    return attrs
       
-class ReadTextAttributes(Task.InputTask):
+class ReadTextColor(_BaseAttributes):
+  '''
+  Read the text foreground and background color names followed by their values.
   '''
-  Read font attributes (size, style, color, etc).
-  '''             
   def execute(self, **kwargs):
     self.stopNow()
-    if not self.sayTextAttrs():
-      self.sayInfo(text=_('default attributes'))
+    # get merged attributes
+    attrs = self._getAttrs()
+
+    # get foreground value
+    fgval = attrs.get('fg-color')
+    fgname = self.getColorString(fgval) or _('default foreground') 
+    
+    # get background value
+    bgval = attrs.get('bg-color')
+    bgname = self.getColorString(bgval) or _('default background')
+
+    # i18n: foreground on background color (e.g. "black on white")
+    template = _('%s on %s.') 
+    names = template % (fgname, bgname)
+    if bgval is None and fgval is None:
+      self.sayColor(text=names)
+    elif bgval is not None and fgval is not None:
+      self.sayColor(text=(fgval, bgval), template=names + ' ' + template)
+    elif fgval is not None:
+      self.sayColor(text=fgval, template=names + ' %s')
+    elif bgval is not None:
+      self.sayColor(text=bgval, template=names + ' %s')
+      
+class ReadTextAttributes(_BaseAttributes):
+  '''
+  Read all remaining text attribute name/value pairs sans color information.
+  
+  @todo: PP: need localizations for all attribute names and text values, or
+    some reasonable subset thereof
+  @todo: PP: probably want names for numbers that have standard levels (e.g. 
+    font weight 400 is bold)
+  '''
+  def execute(self, **kwargs):
+    self.stopNow()
+    attrs = self._getAttrs()
+    for name, value in attrs.items():
+      if not name.endswith('color'):
+        self.sayTextAttrs(text=(name, value), template='%s: %s')
 
 class ReadDescription(Task.InputTask):
   '''
@@ -616,10 +825,16 @@ class ReadItemDetails(Task.Task):
     label = self.getAccLabel()
     if (label != text):
       self.sayItem(text=text)
-    # try to announce the level
-    self.doTask('read new level')
+    # try to announce the headers on the row and column
+    self.doTask('read new headers')
     # see if there are any states worth reporting
     self.sayState()
+    # try to announce the level
+    self.doTask('read new level')
+    # try to announce the item index
+    self.doTask('read item index')
+    # try to announce the total number of items
+    self.doTask('read item count')
     # get the hotkey for this item
     self.sayHotkey()
     if not text:
@@ -695,6 +910,7 @@ class HandleFocusChange(Task.FocusTask):
     self.inhibitMayStop()
     # always reset last caret position
     self.perk.last_caret = POR(item_offset=0)
+    self.perk.last_count = False
     return True
 
 class HandleSelectorChange(Task.SelectorTask):
@@ -712,6 +928,23 @@ class HandleSelectorChange(Task.SelectorTask):
     '''
     self.mayStop()
     self.doTask('read item details', por=por, text=text)
+    
+  def executeText(self, por, text, **kwargs):
+    '''
+    Announces the localized word selected or unselected followed by the total 
+    number of characters in the selection.
+    
+    @param por: Point of regard where the selection event occurred
+    @type por: L{POR}
+    @param text: The currently selected text
+    @type text: string
+    '''
+    n = len(text)
+    if n > self.perk.last_sel_len:
+      self.sayItem(text=n, template=_('selected %d'))
+    else:
+      self.sayItem(text=n, template=_('unselected %d'))
+    self.perk.last_sel_len = n
 
 class HandleCaretChange(Task.CaretTask):
   '''  
@@ -720,9 +953,13 @@ class HandleCaretChange(Task.CaretTask):
   deleted or the caret moved. The other three methods handle these more
   specific cases.
   
-  @todo: PP: think about semantic here for insert, move, and delete; is it echo?
-    is it char/word/item?
-  '''  
+  @ivar changed: Tracks whether the last caret move event was a caused by a
+    change (insert/delete) or just navigation
+  @type changed: boolean
+  '''
+  def init(self):
+    self.changed = False
+  
   def execute(self, por, text, text_offset, added, **kwargs):
     '''
     Stores the point of regard to the caret event location. Tries to stop
@@ -739,9 +976,6 @@ class HandleCaretChange(Task.CaretTask):
       (the default) when event is for caret movement only
     @type added: boolean
     '''
-    #if not self.mayStop() and por.isSameAcc(self.perk.last_caret):
-      ## opt: bail out if we're trying to queue some other text event
-      #return True
     # let the base class call the appropriate method for inset, move, delete
     rv = Task.CaretTask.execute(self, por, text, text_offset, added, **kwargs)
     # store for next comparison
@@ -759,11 +993,29 @@ class HandleCaretChange(Task.CaretTask):
     @param text_offset: The offset of the inserted text
     @type text_offset: integer
     '''
-    self.mayStop()
-    # inhibit the next stop
-    #self.inhibitMayStop()
-    self.sayItem(text=text)
-    return True
+    #print 'CHANGED: insert'
+    self.changed = True
+    if len(text) == 1:
+      w = (self.perk_state.WordEcho and 
+           unicodedata.category(text)[0] in ('C', 'Z'))
+      c = self.perk_state.CharEcho
+      if w and c:
+        # word and character echo
+        self.mayStop()
+        self.sayChar(text=text)
+        self.sayWord(por)
+      elif w:
+        # word echo
+        self.mayStop()
+        self.sayWord(por)
+      elif c:
+        # character echo
+        self.mayStop()
+        self.sayChar(text=text)
+    else:
+      # chunk of text, say the whole thing for now
+      self.mayStop()
+      self.sayItem(text=text)
     
   def executeMoved(self, por, text, text_offset, **kwargs):
     '''
@@ -776,24 +1028,37 @@ class HandleCaretChange(Task.CaretTask):
     @param text_offset: The offset of the new caret position
     @type text_offset: integer
     '''
-    if (not por.isSameItem(self.perk.last_caret) or
-        not por.isSameAcc(self.perk.last_caret)):
-      # say line when item_offset changes
+    # don't echo again if a change just happened
+    if self.changed:
+      #print 'UNCHANGED: moved'
+      self.changed = False
+      return
+    
+    if not por.isSameAcc(self.perk.last_caret):
+      # say the current line
       self.mayStop()
       self.sayItem(text=text)
-    else: 
+      if self.getAccTextSelectionCount() > 0:
+        # and the total number of selected characters if there's a selection
+        self.sayTextAttrs(text=len(self.getAccTextSelection()[0]),
+                          template=_('selected %s'))
+    elif not por.isSameItem(self.perk.last_caret):
+      # say the new line when item_offset changes
+      self.mayStop()
+      self.sayItem(text=text)
+    else:
+      #print 'move:', por, self.perk.last_caret
       # respond to caret movement on same line
       if por.isCharBefore(self.perk.last_caret):
         # moved left, say text between positions
+        diff = text[por.char_offset : self.perk.last_caret.char_offset]
         self.mayStop()
-        self.sayItem(text=text[por.char_offset : 
-                     self.perk.last_caret.char_offset])
+        self.sayItem(text=diff)
       elif por.isCharAfter(self.perk.last_caret):
         # moved right, say text between positions
+        diff = text[self.perk.last_caret.char_offset : por.char_offset]
         self.mayStop()
-        self.sayItem(text=text[self.perk.last_caret.char_offset : 
-                      por.char_offset])
-    return True
+        self.sayItem(text=diff)
     
   def executeDeleted(self, por, text, text_offset, **kwargs):
     '''
@@ -810,14 +1075,14 @@ class HandleCaretChange(Task.CaretTask):
     self.mayStop()
     if (self.perk.last_caret.item_offset + 
         self.perk.last_caret.char_offset) > text_offset:
-      # inhibit the next stop
-      #self.inhibitMayStop()
       # say backspace if we're backing up
       self.sayItem(text=text, template=_('backspace %s'))
+      self.changed = True
+      #print 'CHANGED: delete'
     else:
       # say delete if we're deleting the current
       self.sayItem(text=text, template=_('delete %s'))
-    return True
+      self.changed = False
 
   def update(self, por, text, text_offset, added, **kwargs):
     '''
@@ -831,11 +1096,13 @@ class HandleCaretChange(Task.CaretTask):
       offset when movement only
     @type text_offset: integer
     @param added: True when text added, False when text deleted, and None 
-      (the default) when event is for caret movement only
+      when event is for caret movement only
     @type added: boolean
     '''
     # store for next comparison
     self.perk.last_caret = por
+    self.changed = False
+    #print 'UNCHANGED: update'
   
 class HandleStateChange(Task.StateTask):
   '''  
