@@ -7,18 +7,32 @@ Defines a L{Perk} for the gnome-terminal to correct caret event handling.
 @author: Peter Parente, Brett Clippingdale
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import Task, Perk
 from POR import POR
 
 __uie__ = dict(kind='perk', tier='gnome-terminal')
 
+class GTerminalPerk(Perk.Perk):
+  '''
+  Corrects basic accessibility problems in gnome-terminal. Supplements the 
+  DefaultPerk.
+  '''
+  def init(self):
+    '''
+    Registers a L{HandleCaretChange} task. Initializes an instance variable to
+    store the previous caret L{POR}.
+    '''
+    # set an audio device as the default output
+    self.setPerkIdealOutput('audio')    
+    self.registerTask(HandleCaretChange())
+
 class GTerminalPerk(Perk.Perk):
   '''
   Corrects basic accessibility problems in gnome-terminal. Supplements the 
