@@ -16,7 +16,7 @@ on activate: ["chat with" username]
 on focus: ["compose" or "history"] <active descendant change>
 on message received: [text of message]
 
-@todo: PP: redocument before 0.3 release
+@todo: PP: clean up documentation
   
 @var CONTAINER_PATH: path from frame to container with accessible elements
 @type CONTAINER_PATH: integer
@@ -63,12 +63,12 @@ on message received: [text of message]
 @author: Peter Parente
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import Perk, Task, AEState
 from POR import POR
@@ -110,9 +110,7 @@ STATUS_MSG = [_('unseen'), _('typing'), _('paused'), _('idle'), _('offline'),
 
 class GaimPerkState(Perk.PerkState):
   '''
-  Settings for gaim.
-  
-  @todo: PP: mark _ after 0.3.0
+  Settings for Gaim.
   
   @cvar ReadBackground: Read incoming messages when gaim is in the background?
   @type ReadBackground: boolean
@@ -121,9 +119,9 @@ class GaimPerkState(Perk.PerkState):
   
   def getSettings(self):
     g = self._newGroup()
-    g.newBool('ReadBackground', 'Read background messages?', 'When checked, '
-              'incoming messages will be read even when gaim is in the '
-              'background.')
+    g.newBool('ReadBackground', _('Read background messages?'), 
+              _('When checked, incoming messages will be read even when Gaim '
+                'is in the background.'))
     return g
 
 class GaimPerk(Perk.Perk):
@@ -161,6 +159,9 @@ class GaimPerk(Perk.Perk):
     message, active typing, paused typing) and drive focus back and forth
     between chat composition text area and history text area.
     '''
+    # set an audio device as the default output
+    self.setPerkIdealOutput('audio')    
+    
     # register event handlers
     # events here are app-level, DefaultPerk will handle gnome-level events,
     # so only register events of interest here. 
