@@ -5,15 +5,16 @@ manging L{Tier}s, L{Perk}s, and L{Task}s.
 @author: Peter Parente
 @organization: IBM Corporation
 @copyright: Copyright (c) 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import weakref
 from Error import *
+import AEEvent
 import Base
 import LSRConstants, UIRegistrar
 
@@ -769,14 +770,18 @@ class System(Base.TaskTools):
       mons.removeByClassName(name)
 
   def loadChooser(self, name, **kwargs):
-    '''
+    '''    
     Loads an L{AEChooser} dialog with the given name from disk. Passes the
-    keyword arguments to the init method of the chooser.
+    keyword arguments to the init method of the chooser. This method should
+    typically only be invoked by a L{Task.ChooserTask} subclass.
     
     @param name: UIE name of the chooser
     @type name: string
     @param kwargs: Keyword arguments to initialize the chooser
     @type kwargs: dictionary
+    @return: Reference to the chooser if it was created. If it already existed,
+      None is returned instead.
+    @rtype: L{AEChooser}
     '''
     reg = UIRegistrar
     chooser = reg.loadOne(name)
@@ -788,15 +793,18 @@ class System(Base.TaskTools):
     except ValueError, ex:
       # singleton already exists, reactivate
       ex.args[0].activate(timestamp=self.timestamp, **kwargs)
+      return None
     else:
       # call init to let the chooser initialize itself
       chooser.init(timestamp=self.timestamp, **kwargs)
       # register to handle events from this chooser
       self.perk.registerChooserTask(chooser, self)
+      return chooser
     
   def unloadChooser(self, chooser):
-    '''
-    Unloads the given L{AEChooser} dialog.
+    '''    
+    Unloads the given L{AEChooser} dialog. This method should typically only be
+    invoked by a L{Task.ChooserTask} subclass.
     
     @param chooser: The chooser to unload
     @type chooser: L{AEChooser}
@@ -939,4 +947,24 @@ class System(Base.TaskTools):
     with the active profile.
     '''
     self.device_man.unloadDevices()
-    self.device_man.loadDevices()
\ No newline at end of file
+    self.device_man.loadDevices()
+    
+  def signalSetting(self, state, name, value, **kwargs):
+    '''
+    Sends an L{AEEvent.SettinChange} indicating the setting with the given name
+    in the given L{AEState} object changed its value. Any keyword arguments
+    are provided as-is to observers of the event. The source L{Perk} of the 
+    change is included in the event.
+    
+    @param state: State object having the setting that changed
+    @type state: L{AEState}
+    @param name: Name of the setting variable that changed
+    @type name: string
+    @param value: New value of the setting
+    @type value: object
+    @param kwargs: Arbitrary data to be delivered with the event
+    @type kwargs: dictionary
+    '''
+    event = AEEvent.SettingChange(self.tier.getIdentity(), self.perk, state, 
+                                  name, value, **kwargs)
+    self.event_man.postEvents(event)
\ No newline at end of file
