@@ -7,12 +7,12 @@ application.
 @author: Larry Weiss
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import logging, weakref
 import AEState, AEInput, Perk, Task
@@ -71,6 +71,12 @@ class Tier(object):
     self.pointer_por = por
     self.focus_por = None
 
+  def close(self):
+    '''
+    Clears all L{Perk}s and closes them.
+    '''
+    self.clearPerks()
+
   def getPointer(self):
     '''
     Gets the user L{POR} for this L{Tier}.
@@ -201,15 +207,17 @@ class Tier(object):
     # have to pop in reverse order to avoid corrupting original indices
     indices.sort(reverse=True)
     perks = map(self.perks.pop, indices)
-    map(Perk.Perk.close, perks)
+    for perk in self.perks:
+      perk.close()
     map(Perk.Perk.postClose, perks)
-      
+
   def clearPerks(self):
     '''
     Removes all L{Perk}s from the stack. Calls L{Perk.Perk.close} on each to 
     ensure they have a chance to persist state.
     '''
-    map(Perk.Perk.close, self.perks)
+    for perk in self.perks:
+      perk.close()
     map(Perk.Perk.postClose, self.perks)
     self.perks = []
       
@@ -440,7 +448,7 @@ class Tier(object):
         self.tier_manager.showTask(event, perk, task, propagate)
       perk.postExecute(False)
         
-  def manageGesture(self, event):
+  def manageGesture(self, event, count):
     '''
     Manages an event by getting the L{AEInput.Gesture} that triggered it and
     locating a L{Task} registered to execute in response to it. If a L{Task} 
@@ -449,6 +457,9 @@ class Tier(object):
 
     @param event: Event to process
     @type event: L{AEEvent.InputGesture}
+    @param count: Number of times this gesture has been issued without 
+      interruption
+    @type count: integer
     '''
     # look for a Perk with a Task registered to respond to this input command
     task = self.getCommandTask(event.getTaskKey())
@@ -461,7 +472,7 @@ class Tier(object):
       if not perk.preExecute(layer):
         return
       # execute that Task
-      self._executeTask(task, None, layer, {}, True, timestamp)
+      self._executeTask(task, None, layer, {'count' : count}, True, timestamp)
       perk.postExecute(False)
       
   def manageKeyedTask(self, event):
@@ -513,4 +524,4 @@ class Tier(object):
         return
       # execute that Task
       self._executeTask(task, por, layer, task_data, True)
-      perk.postExecute(False)
\ No newline at end of file
+      perk.postExecute(False)
